package com.zswl.cloud.springBtach.server.core.publish;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.github.microservice.productcenter.client.model.LocationModel;
import com.github.microservice.productcenter.client.model.ProductPushModel;
import com.github.microservice.productcenter.client.model.ProductTriggerTaskModel;
import com.github.microservice.productcenter.client.model.RegionModel;
import com.github.microservice.productcenter.client.publish.ProductPublish;
import com.github.microservice.productcenter.client.service.ProductPushService;
import com.github.microservice.productcenter.client.stream.ProductCenterStream;
import com.github.microservice.productcenter.client.type.ProductType;
import com.github.microservice.productcenter.client.type.RegionType;
import com.zswl.cloud.springBatch.client.model.ypp.kfc.response.CityListResponse;
import com.zswl.cloud.springBatch.client.model.ypp.kfc.response.ShopsResponse;
import com.zswl.cloud.springBtach.server.core.api.ypp.KfcApi;
import com.zswl.cloud.springBtach.server.core.api.ypp.request.KfcShopsRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
public class KfcPublish implements ProductPublish {

    @Autowired
    KfcApi kfcApi;


    @Autowired
    private ProductCenterStream productCenterStream;

    @Override
    public String regionCode() {
        return null;
    }

    @Override
    public Collection<RegionType> regionType() {
        return Set.of(RegionType.City);
    }

    @Override
    public Collection<ProductType> productTypes() {
        return Set.of(
                ProductType.Restaurant
        );
    }


    @Override
    public void trigger(ProductPushService updateService, ProductTriggerTaskModel productTriggerTask) {


        CityListResponse cityListResponse = kfcApi.cityList();
        Map<String, String> collect = cityListResponse.getData().stream().collect(Collectors.toMap(it -> it.getName(), it -> it.getCityId()));

        Object updateName = productTriggerTask.getRegion().getParams().get("updateName");
        String name = String.valueOf(updateName);
        String s = collect.get(name);
        if (ObjectUtil.isEmpty(s)) {
            throw new RuntimeException("地区信息不匹配");
        }

        ArrayList<String> strings = new ArrayList<>();
        strings.add("Kfc");
        strings.add("MDL");
        strings.add("XBK");
        strings.add("NXDC");
        strings.add("RXKF");
        strings.add("XC");
        strings.add("KD");

        for (String string : strings) {
            KfcShopsRequest kfcShopsRequest = new KfcShopsRequest();
            kfcShopsRequest.setCityId(s);
            kfcShopsRequest.setOt(string);
            ShopsResponse shops = kfcApi.shops(kfcShopsRequest);
            shops.getData().forEach(it -> {
                        LocationModel locationModel = new LocationModel();
                        try {

                            locationModel.setLongitude(it.getLon());
                            locationModel.setLatitude(it.getLat());

                            productCenterStream.send(List.of(
                                    ProductPushModel.builder()
                                            .regionCode(productTriggerTask.getRegion().getCode())
                                            .pid(it.getId())
                                            .address(it.getAddress())
                                            .parkId(productTriggerTask.getRegion().getParentId())
                                            .productType(ProductType.Restaurant)
                                            .locationPoint(locationModel)
                                            .title(it.getShopName())
                                            .meta(Map.of("shop", it, "ot", string))
                                            .expireTimeSecond(60 * 60 * 24 * 10)
                                            .build()));
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                        try {
                            JSONObject menus = kfcApi.menus(it.getId());
                            ArrayList arrayList = (ArrayList) menus.getByPath("data.menuVoList");
                            arrayList.forEach(list -> {

                                JSONArray jsonArray = new JSONArray(list);
                                jsonArray.forEach(item -> {
                                    JSONObject jsonObject = (JSONObject) item;
                                    Object productId = jsonObject.get("productId");
                                    Object nameCn = jsonObject.get("nameCn");
                                    Object priceHead = jsonObject.get("priceHead");
                                    Object imageUrl = jsonObject.get("imageUrl");
                                    jsonObject.set("shop", it);
                                    productCenterStream.send(List.of(
                                            ProductPushModel.builder()
                                                    .regionCode(productTriggerTask.getRegion().getCode())
                                                    .parkId(productTriggerTask.getRegion().getParentId())
                                                    .pid((String) productId)
                                                    .address(it.getAddress())
                                                    .productType(ProductType.RestaurantFood)
                                                    .locationPoint(locationModel)
                                                    .title((String) nameCn)
                                                    .cover(new String[]{(String) imageUrl})
                                                    .salePrice(new BigDecimal(String.valueOf(priceHead)).multiply(BigDecimal.valueOf(100)).longValue())
                                                    .meta(Map.of("menus", jsonObject, "ot", string))
                                                    .expireTimeSecond(60*60*24)
                                                    .build()));

                                });
                            });


                        } catch (Exception e) {
                        }

                    }

            );
        }
    }
}
