package com.zswl.cloud.springBtach.server.core.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.eeext.openapi.entity.*;
import com.github.microservice.auth.client.content.ResultContent;
import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.auth.security.helper.AuthHelper;
import com.zhongshu.payment.client.model.order.v2.*;
import com.zhongshu.payment.client.service.v2.OrderService2;
import com.zhongshu.payment.client.service.v2.PaymentService2;
import com.zhongshu.payment.client.type.*;
import com.zswl.cloud.shop.client.service.CouponInfoLogService;
import com.zswl.cloud.shop.client.service.ShopService;
import com.zswl.cloud.shop.client.vo.life.AccountDetailVo;
import com.zswl.cloud.shop.client.vo.life.SharePlatformVo;
import com.zswl.cloud.shop.client.vo.life.ShopInfoSimpleVo;
import com.zswl.cloud.springBatch.client.model.video2.RechargeRequest;
import com.zswl.cloud.springBatch.client.service.Video2Service;
import com.zswl.cloud.springBtach.server.core.api.video2.Video2Api;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Log4j2
public class Video2ServiceImpl implements Video2Service {


    @Autowired
    private Video2Api video2Api;

    @Autowired
    ShopService shopService;

    @Autowired
    Snowflake snowflake;

    @Autowired
    AuthHelper authHelper;

    @Autowired
    OrderService2 orderService2;
    @Autowired
    private PaymentService2 paymentService2;

    @Autowired
    CouponInfoLogService couponInfoLogService;

    @Value("${video2.notifyUrl}")
    private String notifyUrl;

    /**
     * 商品列表
     *
     * @return
     */
    public ResultContent<RechargeProductResp> rechargeProduct() {

        return ResultContent.buildContent(video2Api.rechargeProduct());
    }

    public ResultContent<CreatOrderResultModel> creatOrder(RechargeRequest rechargeOrderReq) {

        RechargeProductResp rechargeProductResp = video2Api.rechargeProduct();
        List<ProductItemResp> collect = rechargeProductResp.getProducts().stream().filter(productItemResp -> productItemResp.getProductId().equals(rechargeOrderReq.getProductId())).collect(Collectors.toList());
        if (collect.isEmpty()) {
            return ResultContent.build(ResultState.Fail, "视频会员不存在");
        }
        ProductItemResp productItemResp = collect.get(0);
        String channelPrice = productItemResp.getChannelPrice();
        BigDecimal totalPrice = new BigDecimal(channelPrice);
        com.zswl.cloud.shop.client.ret.ResultContent<AccountDetailVo> shopSetting = shopService.shopSetting("视频会员",rechargeOrderReq.getCode());
        if (!shopSetting.getState().equals(com.zswl.cloud.shop.client.ret.ResultState.Success)) {
            return ResultContent.build(ResultState.Fail, "视频会员信息未配置");
        }
        List<ShopInfoSimpleVo> shops = shopSetting.getContent().getShops();
        if (shops.isEmpty()) {
            return ResultContent.build(ResultState.Fail, "视频会员店铺未创建");
        }
        AccountDetailVo accountDetailVo = shopSetting.getContent();


        String orderNo = snowflake.nextIdStr();

// 平台订单

        ShopInfoSimpleVo shopInfoSimpleVo = shops.get(0);
        String shopId = shopInfoSimpleVo.getShopId();
        String authAccount = accountDetailVo.getAuthAccount();
        SharePlatformVo sharePlatformVo = accountDetailVo.getSharePlatformVo();

        // 平台订单
        CreateOrder2Model createOrder2Model = new CreateOrder2Model();
        createOrder2Model.setOrderNo(orderNo);
        createOrder2Model.setUserId(authHelper.getCurrentUser().getUserId());
        createOrder2Model.setShopId(shopId);
        JSONObject shopInfo = JSONUtil.parseObj(shopSetting);
        shopInfo.set("shopName", shopInfoSimpleVo.getShopName());
        createOrder2Model.setShopInfo(shopInfo);
        createOrder2Model.setTotalAmount(totalPrice);
        createOrder2Model.setDiscountAmount(BigDecimal.ZERO);
        createOrder2Model.setPayAmount(totalPrice);
        createOrder2Model.setAccesserUserId(authAccount);
        createOrder2Model.setCode(rechargeOrderReq.getCode());
        JSONObject goodsInfo = JSONUtil.parseObj(rechargeOrderReq);
        goodsInfo.set("goodsName", productItemResp.getItemName());
//        goodsInfo.set("goodsPath", addOrderRequest.getRoomPic());

        CreateOrder2Model.Goods goods = new CreateOrder2Model.Goods();
        goods.setGoodsId(productItemResp.getProductId().toString());
        goods.setJobFlowMap(JobFlowMap.video2);
        goods.setGoodsInfo(goodsInfo);
        goods.setPayAmount(totalPrice);
        goods.setValidDay(90);
        goods.setTransferType(sharePlatformVo.getShareType().equals(1) ? TransferType.RATIO : TransferType.FIXED);

        List<CreateOrder2Model.Owner> ownerList = new ArrayList<>();

        CreateOrder2Model.Owner platfor = new CreateOrder2Model.Owner();
        platfor.setOwner(TransferOwner.PLATFORM);
        platfor.setValue(sharePlatformVo.getPlatformRate());

        CreateOrder2Model.Owner buy = new CreateOrder2Model.Owner();
        buy.setOwner(TransferOwner.BUY_SHOP);
        buy.setValue(sharePlatformVo.getBuyShopRate());
        buy.setShopId(shopId);

        CreateOrder2Model.Owner use = new CreateOrder2Model.Owner();
        use.setOwner(TransferOwner.USE_SHOP);
        use.setValue(sharePlatformVo.getCheckShopRate());

        ownerList.add(platfor);
        ownerList.add(buy);
        ownerList.add(use);
        goods.setTransferRrule(ownerList);
        createOrder2Model.setGoodsList(List.of(goods));
        return orderService2.creat(createOrder2Model);

    }

    public void notifyOrder(NotifyOrderReq notifyOrderReq) {
        ResultContent<List<GoodsResultModel>> listResultContent = orderService2.goodsDetail(notifyOrderReq.getOutTradeNo());
        GoodsResultModel goodsResultModel = listResultContent.getContent().get(0);
        UpdateGoodsModel updateGoodsModel = new UpdateGoodsModel();
        updateGoodsModel.setId(goodsResultModel.getId());
        if (notifyOrderReq.getStatus().equals("01")) {
            updateGoodsModel.setGoodsState(OrderType.USED);
        } else if (notifyOrderReq.getStatus().equals("03")) {
            couponInfoLogService.rollback(notifyOrderReq.getOutTradeNo());
            if (goodsResultModel.getPayAmount().equals(BigDecimal.ZERO)) {
                updateGoodsModel.setGoodsState(OrderType.CLOSE);
            } else {
                ApplicantRefundParam2Model applicantRefundParam2Model = new ApplicantRefundParam2Model();
                applicantRefundParam2Model.setId(goodsResultModel.getId());
                applicantRefundParam2Model.setRemark("充值失败退款");
                applicantRefundParam2Model.setUserId(goodsResultModel.getUserId());
                paymentService2.refund(applicantRefundParam2Model);
            }
        }
        updateGoodsModel.setExtend(JSONUtil.parseObj(notifyOrderReq));
        orderService2.updateGoods(updateGoodsModel);

    }


    public ResultContent<String> rechargeOrder(RechargeRequest rechargeRequest) {
        RechargeOrderReq rechargeOrderReq = new RechargeOrderReq();
        BeanUtils.copyProperties(rechargeRequest, rechargeOrderReq);
        rechargeOrderReq.setNotifyUrl(notifyUrl);
        rechargeOrderReq.setNumber(1);


        RechargeProductResp rechargeProductResp = video2Api.rechargeProduct();
        List<ProductItemResp> collect = rechargeProductResp.getProducts().stream().filter(productItemResp -> productItemResp.getProductId().equals(rechargeOrderReq.getProductId())).collect(Collectors.toList());
        if (collect.isEmpty()) {
            return ResultContent.build(ResultState.Fail, "视频会员不存在");
        }
        ProductItemResp productItemResp = collect.get(0);
        BigDecimal totalPrice = BigDecimal.ZERO;
        com.zswl.cloud.shop.client.ret.ResultContent<AccountDetailVo> shopSetting = shopService.shopSetting("视频会员",rechargeRequest.getCode());
        if (!shopSetting.getState().equals(com.zswl.cloud.shop.client.ret.ResultState.Success)) {
            return ResultContent.build(ResultState.Fail, "视频会员信息未配置");
        }

        String orderNo = snowflake.nextIdStr();
        rechargeOrderReq.setOutTradeNo(orderNo);
// 平台订单
        AccountDetailVo accountDetailVo = shopSetting.getContent();
        String shopId = accountDetailVo.getAccountId();
        String authAccount = accountDetailVo.getAuthAccount();
        SharePlatformVo sharePlatformVo = accountDetailVo.getSharePlatformVo();

        // 平台订单
        CreateOrder2Model createOrder2Model = new CreateOrder2Model();
        createOrder2Model.setOrderNo(orderNo);
        createOrder2Model.setUserId(rechargeRequest.getUserId());
        createOrder2Model.setShopId(shopId);
        JSONObject shopInfo = JSONUtil.parseObj(shopSetting);
        shopInfo.set("shopName", "会员充值");
        createOrder2Model.setShopInfo(shopInfo);
        createOrder2Model.setTotalAmount(totalPrice);
        createOrder2Model.setDiscountAmount(BigDecimal.ZERO);
        createOrder2Model.setPayAmount(totalPrice);
        createOrder2Model.setAccesserUserId(authAccount);
        createOrder2Model.setCode(rechargeRequest.getCode());
        JSONObject goodsInfo = JSONUtil.parseObj(rechargeOrderReq);
        goodsInfo.set("goodsName", productItemResp.getItemName());

        CreateOrder2Model.Goods goods = new CreateOrder2Model.Goods();
        goods.setGoodsId(productItemResp.getProductId().toString());
        goods.setJobFlowMap(JobFlowMap.video2);
        goods.setGoodsInfo(goodsInfo);
        goods.setPayAmount(totalPrice);
        goods.setValidDay(90);
        goods.setTransferType(sharePlatformVo.getShareType().equals(1) ? TransferType.RATIO : TransferType.FIXED);

        List<CreateOrder2Model.Owner> ownerList = new ArrayList<>();

        CreateOrder2Model.Owner platfor = new CreateOrder2Model.Owner();
        platfor.setOwner(TransferOwner.PLATFORM);
        platfor.setValue(sharePlatformVo.getPlatformRate());

        CreateOrder2Model.Owner buy = new CreateOrder2Model.Owner();
        buy.setOwner(TransferOwner.BUY_SHOP);
        buy.setValue(sharePlatformVo.getBuyShopRate());
        buy.setShopId(shopId);

        CreateOrder2Model.Owner use = new CreateOrder2Model.Owner();
        use.setOwner(TransferOwner.USE_SHOP);
        use.setValue(sharePlatformVo.getCheckShopRate());

        ownerList.add(platfor);
        ownerList.add(buy);
        ownerList.add(use);
        goods.setTransferRrule(ownerList);
        createOrder2Model.setGoodsList(List.of(goods));
        ResultContent<CreatOrderResultModel> creat = orderService2.creat(createOrder2Model);
        if (!creat.getState().equals(ResultState.Success)) {

            return ResultContent.build(ResultState.Fail, "创建订单失败");
        }
        RechargeOrderResp rechargeOrderResp = video2Api.rechargeOrder(rechargeOrderReq);

        if (!rechargeOrderResp.isSuccess()) {
            return ResultContent.buildContent(rechargeOrderResp.getMessage());
        }
        return ResultContent.buildContent(orderNo);
    }

    public ResultContent<RechargeQueryResp> rechargeQuery(String outTradeNo) {

        return ResultContent.buildContent(video2Api.rechargeQuery(outTradeNo));
    }
}
